package org.cobbzilla.wizard.model.entityconfig.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Build Entity-config for the annotated class with `baseURI` as list and create URIs, and with appended
 * `identifierInURI` (automatically surrounded with curled brackets) for update and delete URIs. Default value for
 * `identifierInURI` is `uuid`.
 *
 * Also, each URI may be set to not defined with corresponding `is...Defined` boolean value. By default all (list,
 * create, update and delete) URIs will be defined with this annotation.
 *
 * Each usage of other `@ECType...` annotation will override corresponding config generated by this annotation.
 */
@Retention(RetentionPolicy.RUNTIME) @Target(ElementType.TYPE)
public @interface ECTypeURIs {
    String baseURI() default ""; // URIs for create and list most of the time.
    String identifierInURI() default "uuid"; // column name used to identify object in update and delete URIs (we got
                                             // from baseURI)

    String[] listFields() default {};

    // So of the actions are not defined at all - set these flags according to that. Note that URI_NOT_SUPPORTED is a
    // different thing, and should be redefined with a more specific ECType annotation.
    boolean isListDefined() default true;
    boolean isCreateDefined() default true;
    boolean isUpdateDefined() default true;
    boolean isDeleteDefined() default true;
}
